package org.fouram.core.plugin.weixin.cp.util;

import static org.fouram.core.plugin.weixin.cp.core.WXCpBase.departService;

import java.util.List;
import java.util.Map;

import org.fouram.core.util.LoggerUtil;
import org.fouram.core.util.Tools;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.bean.WxCpDepart;

/**
 * 企业微信科室工具类
 * 
 * @author Liup
 */
public class WXCpDepartUtil {

	public static List<WxCpDepart> getChildDeparts(Long id) throws WxErrorException {
		List<WxCpDepart> departs = Lists.newArrayList();
		for (WxCpDepart depart : departService.list(id)) {
			if (id.equals(depart.getParentId())) {
				departs.add(depart);
			}
		}
		return departs;
	}

	public static Map<String, Long> getChildDepartMap(Long id) throws WxErrorException {
		Map<String, Long> map = Maps.newHashMap();
		for (WxCpDepart depart : departService.list(id)) {
			if (id.equals(depart.getParentId())) {
				map.put(depart.getName(), depart.getId());
			}
		}
		return map;
	}

	public static Long create(String subCompany, String depatment) throws WxErrorException {
		Long subCompanyId = create(1L, subCompany);
		if (Tools.isEmpty(depatment)) {
			return subCompanyId;
		}
		return create(subCompanyId, depatment);
	}

	public static Long create(String firstOrg, String secondOrg, String thirdOrg, String fourthOrg)
			throws WxErrorException {
		Long firstOrgId = create(1L, firstOrg);
		if (Tools.isEmpty(secondOrg)) {
			return firstOrgId;
		}
		Long secondOrgId = create(firstOrgId, secondOrg);
		if (Tools.isEmpty(thirdOrg)) {
			return secondOrgId;
		}
		Long thirdOrgId = create(secondOrgId, thirdOrg);
		if (Tools.isEmpty(fourthOrg)) {
			return thirdOrgId;
		}
		return create(thirdOrgId, fourthOrg);
	}

	public static Long create(Long firstOrgWxDepartId, Long secondOrgWxDepartId, Long thirdOrgWxDepartId,
			Long fourthOrgWxDepartId) throws WxErrorException {
		if (secondOrgWxDepartId == null) {
			return firstOrgWxDepartId;
		}
		if (thirdOrgWxDepartId == null) {
			return secondOrgWxDepartId;
		}
		if (fourthOrgWxDepartId == null) {
			return thirdOrgWxDepartId;
		}
		return fourthOrgWxDepartId;
	}

	private static Long create(Long parentId, String depatment) throws WxErrorException {
		Map<String, Long> map = getChildDepartMap(parentId);
		Long depatmentId = map.get(depatment);
		if (depatmentId == null) {
			WxCpDepart depart = new WxCpDepart();
			depart.setName(depatment);
			depart.setParentId(parentId);
			depatmentId = departService.create(depart);
		}
		return depatmentId;
	}
	
	public static Long createOrUpdate(Long parentId, String name) throws WxErrorException {
		Map<String, Long> map = getChildDepartMap(parentId);
		LoggerUtil.info("createOrUpdate->" + JSON.toJSONString(map));
		Long wxDepartId = map.get(name);
		// 1.判断parentId对应子部门是否存在对应部门，存在返回对应
		// 2.不存在创建对应子部门
		if(wxDepartId == null) {
			WxCpDepart depart = new WxCpDepart();
			depart.setName(name);
			depart.setParentId(parentId);
			wxDepartId = departService.create(depart);
		}
		return wxDepartId;
	}

	public static List<WxCpDepart> getAllChildDeparts(Long id) throws WxErrorException {
		return departService.list(id);
	}

	public static void delete(Long id) throws WxErrorException {
		departService.delete(id);
	}
}
