package org.fouram.job;

import org.fouram.core.plugin.weixin.cp.util.WXCpMessageUtil;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.core.util.Tools;
import org.fouram.service.SapCheckdateService;
import org.fouram.service.SapEmployeeTimeService;
import org.fouram.service.SapUserService;
import org.fouram.service.WxApprovalInfoService;
import org.fouram.service.WxCheckOptionService;
import org.fouram.service.WxCheckService;
import org.fouram.service.WxDepartService;
import org.fouram.service.WxUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class CheckTaskSchedule {

	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private WxCheckService wxCheckService;
	@Autowired
	private SapUserService sapUserService;
	@Autowired
	private WxDepartService wxDepartService;
	@Autowired
	private SapCheckdateService sapCheckdateService;
	@Autowired
	private WxCheckOptionService wxCheckOptionService;
	@Autowired
	private SapEmployeeTimeService employeeTimeService;
	@Autowired
	private WxApprovalInfoService wxApprovalInfoService;

	public void taskBiz() throws Exception {
		LoggerUtil.info(DateUtil.getTime());
		try {
			if (DateUtil.getHour().equals("10") || DateUtil.getHour().equals("12") || DateUtil.getHour().equals("20")
					|| DateUtil.getHour().equals("23")) {
				LoggerUtil.info("执行开始:" + DateUtil.getTime());
				executeBiz(DateUtil.getDay(), DateUtil.getHour());
				LoggerUtil.info("执行结束:" + DateUtil.getTime());
			}
		} catch (Exception e) {
			WXCpMessageUtil.sendAdminError(Tools.getExceptionAllInfo(e));
			LoggerUtil.error(e);
		}
	}

	@Transactional
	public void executeBiz(String checkinDay, String hour) throws Exception {
		String startDay = checkinDay;
		String endDay = checkinDay;
		wxDepartService.saveWxDeparts();
		LoggerUtil.info("saveWxDeparts:" + DateUtil.getTime());
		wxUserService.saveWxUsers();
		LoggerUtil.info("saveWxUsers:" + DateUtil.getTime());
		sapUserService.saveOrUpdateSapUsers();
		LoggerUtil.info("saveOrUpdateSapUsers:" + DateUtil.getTime());
		wxUserService.updateAllSapUserId();
		LoggerUtil.info("updateAllSapUserIds:" + DateUtil.getTime());
		wxCheckService.saveCheckInOffOuts(startDay, endDay);
		LoggerUtil.info("saveCheckInOffOuts:" + DateUtil.getTime());
		// 判断打卡记录小于10条记录认为是假期
		int checkInTotal = wxCheckService.queryCheckInTotal().intValue();
		if (checkInTotal < 10) {
			return;
		}
		sapCheckdateService.saveCheckdate(startDay);
		LoggerUtil.info("saveCheckout:" + DateUtil.getTime());
		employeeTimeService.saveEmployeeTimes();
		LoggerUtil.info("saveEmployeeTimes:" + DateUtil.getTime());
		wxCheckOptionService.saveOptions(endDay);
		LoggerUtil.info("saveOptions:" + DateUtil.getTime());
		wxApprovalInfoService.saveApprovalInfos(endDay.substring(0, 7));
		LoggerUtil.info("saveApprovalInfos:" + DateUtil.getTime());
		boolean isSpecialCheckinDay = true;
		if (checkInTotal < 50) {
			isSpecialCheckinDay = false;
		}
		wxCheckService.saveReportResultData(startDay, endDay, isSpecialCheckinDay);
		LoggerUtil.info("saveReportResultData:" + DateUtil.getTime());
		wxCheckService.updateReportResultData(startDay, endDay, hour);
		LoggerUtil.info("updateReportResultData:" + DateUtil.getTime());
	}
}
