package org.fouram.service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.fouram.core.base.service.BaseService;
import org.fouram.core.plugin.weixin.cp.util.WXCpOaUtil;
import org.fouram.core.util.BeanUtils;
import org.fouram.core.util.DateUtil;
import org.fouram.entity.SapCheck;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

import me.chanjar.weixin.cp.bean.oa.WxCpCheckinData;

@Service
public class WxCheckService extends BaseService {

	@Autowired
	private WxUserService wxUserService;

	public void saveCheckInOffOuts(String startDay, String endDay) throws Exception {
		// 重新生成制定日期范围内的打卡数据
		update("SapCheckMapper.deleteCheckinList", null);
		update("SapCheckMapper.deleteCheckoffList", null);
		update("SapCheckMapper.deleteCheckoutList", null);

		Date startDate = DateUtil.sdfTime.parse(startDay + " 00:00:00");
		Date endDate = DateUtil.sdfTime.parse(endDay + " 23:59:59");

		List<String> userIds = wxUserService.selectAllUserIds();
		Map<String, List<String>> map = groupList(userIds, 100);
		for (String key : map.keySet()) {
			List<WxCpCheckinData> checkinDatas = WXCpOaUtil.getCheckinData(startDate, endDate, map.get(key));
			for (WxCpCheckinData data : checkinDatas) {
				SapCheck check = new SapCheck();
				BeanUtils.copyProperties(data, check);
				check.setCheckinDate(new Date(check.getCheckinTime() * 1000));
				check.setCheckinDay(DateUtil.getDay(check.getCheckinDate()));
				if (data.getCheckinType().equals("上班打卡")) {
					saveCheckin(check);
				} else if (data.getCheckinType().equals("下班打卡")) {
					saveCheckoff(check);
				} else if (data.getCheckinType().equals("外出打卡")) {
					saveCheckout(check);
				} else {
					System.out.println(data);
				}
			}
		}

		// 生成制定日期每天外出打开开始和结束时间
		update("SapCheckMapper.deleteCheckoutStartList", null);
		update("SapCheckMapper.deleteCheckoutEndList", null);
		update("SapCheckMapper.saveCheckoutStart1", null);
		update("SapCheckMapper.saveCheckoutStart2", null);
		update("SapCheckMapper.saveCheckoutEnd", null);

		// 删除重复数据
		updateRepeatCheckDelete();
	}

	public Long queryCheckInTotal() throws Exception {
		return (Long) findObject("SapCheckMapper.queryCheckInTotal", null);
	}

	public void saveCheckin(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckin", check);
	}

	public void saveCheckoff(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckoff", check);
	}

	public void saveCheckout(SapCheck check) throws Exception {
		save("SapCheckMapper.saveCheckout", check);
	}

	@SuppressWarnings("unchecked")
	public void updateRepeatCheckDelete() throws Exception {
		// 处理重复数据
		List<SapCheck> sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckin", null);
		Set<String> checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckinById", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoff", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoffById", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoutStart", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoutStartById", sapCheck);
			}
		}

		sapChecks = (List<SapCheck>) findList("SapCheckMapper.selectRepeatCheckoutEnd", null);
		checkinDays = Sets.newHashSet();
		for (SapCheck sapCheck : sapChecks) {
			if (!checkinDays.contains(sapCheck.getCheckinDay() + sapCheck.getUserId())) {
				checkinDays.add(sapCheck.getCheckinDay() + sapCheck.getUserId());
			} else {
				update("SapCheckMapper.deleteCheckoutEndById", sapCheck);
			}
		}
	}

	public void saveReportResultData(String startDay, String endDay, boolean isSpecialCheckinDay) throws Exception {
		Map<String, Object> param = Maps.newHashMap();
		param.put("startDay", startDay);
		param.put("endDay", endDay);
		param.put("startDayT", startDay.replace("-", ""));
		param.put("endDayT", endDay.replace("-", ""));
		param.put("isSpecialCheckinDay", isSpecialCheckinDay);

		// 生成报表数据
		update("SapCheckMapper.deleteCheckReport", param);
		update("SapCheckMapper.saveCheckReport", param);
	}

	public void updateReportResultData(String startDay, String endDay, String hour) throws Exception {
		Map<String, String> param = Maps.newHashMap();
		param.put("hour", hour);
		param.put("startDay", startDay);
		param.put("endDay", endDay);
		param.put("startDayT", startDay.replace("-", ""));
		param.put("endDayT", endDay.replace("-", ""));

		// 更新打卡规则时间
		update("SapCheckMapper.updateWorkTime", param);
		update("SapCheckMapper.updateOffWorkTime", param);
		// 更新noNeedOffwork
		update("SapCheckMapper.updateNoNeedOffwork", param);
		update("SapCheckMapper.updateNoNeedOffworkDefault", param);
		// 更新时间为空的为后台更新未打卡
		update("SapCheckMapper.updateCheckinNullException", param);
		update("SapCheckMapper.updateCheckoffNullException", param);
		// 更新补卡日期
		update("SapCheckMapper.updateBuKaDay", param);
		// 查询请假日期
		update("SapCheckMapper.updateLeaveTime", param);
		// 更新hour字段
		update("SapCheckMapper.updateAllHour", param);
		// 新增列：上班时间
		// 如“固定打卡”未打，“最早外出打卡时间”有则取“最早外出打卡时间”作为“上班时间”
		// 如“固定打卡”or“最早外出打卡时间”都有则取“固定时间”作为“上班时间”
		// “最早外出打卡时间”在“13:00”前才能作为“上班时间”
		update("SapCheckMapper.updateRealCheckinDate", param);
		// 新增列：下班时间
		// 如有“固定打卡时间”or“最晚外出打卡时间”则取“最晚外出打卡时间”作为“下班时间”
		// “最晚外出打卡时间”在“12:00”后才能作为“下班时间”
		update("SapCheckMapper.updateRealCheckoffDate", param);
		// 不需要下班打卡更新异常字段
		update("SapCheckMapper.updateCheckoffExceptionByNoNeedOffwork", param);
		// 异常信息提示
		// 1.列“上班打卡异常信息”
		// a.当“上班时间”为空，列“上班打卡时间”置为空值(现在会显示打卡规则时间)，提示“上班未打卡”
		// b.当“上班时间”晚于打卡规则，提示“迟到”
		// 2.列“下班打卡异常信息”
		// a.当“下班时间”为空，列“下班打卡时间”置为空值，提示“下班未打卡”
		// b.当“下班时间”晚于打卡规则，提示“早退”
		// 增加列：“备注”，备注显示信息如下：符合多种状态逗号分割显示
		update("SapCheckMapper.updateRealResultRemark", param);
		update("SapCheckMapper.updateRealResult", param);
		update("SapCheckMapper.updateRealResultRemarkOK", param);
		update("SapCheckMapper.updateRealResultRemarkLeave", param);
		// 更新工作时间（分钟）
		update("SapCheckMapper.updateWorkMins", param);
		update("SapCheckMapper.updateWorkMinsZero", param);
		update("SapCheckMapper.updateWorkHours", param);
	}
}